'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = shallow;

var _vue = require('vue');

var _vue2 = _interopRequireDefault(_vue);

var _lodash = require('lodash');

var _mount = require('./mount');

var _mount2 = _interopRequireDefault(_mount);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var LIFECYCLE_HOOKS = ['beforeCreate', 'created', 'beforeMount', 'mounted', 'beforeUpdate', 'updated', 'beforeDestroy', 'destroyed', 'activated', 'deactivated'];

function stubLifeCycleEvents(component) {
  LIFECYCLE_HOOKS.forEach(function (hook) {
    component[hook] = function () {};
  });
}
function isRequired(name) {
  return name === 'KeepAlive' || name === 'Transition' || name === 'TransitionGroup';
}

function replaceGlobalComponents(instance, component) {
  Object.keys(instance.options.components).forEach(function (c) {
    if (isRequired(c)) {
      return;
    }
    if (!component.components) {
      component.components = {};
    }
    component.components[c] = {
      render: function render() {},
      attrs: instance.options.components[c].attrs,
      name: instance.options.components[c].name,
      on: instance.options.components[c].on,
      key: instance.options.components[c].key,
      ref: instance.options.components[c].ref,
      props: instance.options.components[c].props,
      domProps: instance.options.components[c].domProps,
      class: instance.options.components[c].class
    };
    delete component.components[c]._Ctor;
    stubLifeCycleEvents(component.components[c]);
    stubLifeCycleEvents(instance.options.components[c]);
  });
}

function replaceComponents(component) {
  Object.keys(component.components).forEach(function (c) {
    delete component.components[c]._Ctor;
    component.components[c] = {
      attrs: component.components[c].attrs,
      name: component.components[c].name,
      on: component.components[c].on,
      key: component.components[c].key,
      ref: component.components[c].ref,
      props: component.components[c].props,
      domProps: component.components[c].domProps,
      class: component.components[c].class,
      render: function render() {}
    };
    _vue2.default.config.ignoredElements.push(c);
    stubLifeCycleEvents(component.components[c]);
  });
}

function shallow(component, options) {
  var clonedComponent = (0, _lodash.cloneDeep)(component);
  if (clonedComponent.components) {
    replaceComponents(clonedComponent);
  }
  replaceGlobalComponents(_vue2.default, clonedComponent);

  return (0, _mount2.default)(clonedComponent, options);
}